@@ -1,14 +1,90 @@
 import re
+import uuid
 
 import django
+from django.db import models
 from django.db.models import Case, Count, Q, When
-from django.test import TestCase
+from django.test import TestCase, TransactionTestCase
 from django.utils import six
-from polymorphic.contrib.guardian import get_polymorphic_base_content_type
-from polymorphic.tests import *  # all models
+
+from polymorphic.managers import PolymorphicManager
+from polymorphic.models import PolymorphicTypeUndefined
+from polymorphic.tests.models import (
+    ArtProject,
+    Base,
+    BlogA,
+    BlogB,
+    BlogBase,
+    BlogEntry,
+    BlogEntry_limit_choices_to,
+    ChildModelWithManager,
+    CustomPkBase,
+    CustomPkInherit,
+    Duck,
+    Enhance_Base,
+    Enhance_Inherit,
+    InitTestModelSubclass,
+    MRODerived,
+    Model2A,
+    Model2B,
+    Model2C,
+    Model2D,
+    ModelExtraA,
+    ModelExtraB,
+    ModelExtraC,
+    ModelExtraExternal,
+    ModelFieldNameTest,
+    ModelShow1,
+    ModelShow1_plain,
+    ModelShow2,
+    ModelShow2_plain,
+    ModelShow3,
+    ModelUnderRelChild,
+    ModelUnderRelParent,
+    ModelWithMyManager,
+    ModelWithMyManager2,
+    ModelWithMyManagerDefault,
+    ModelWithMyManagerNoDefault,
+    ModelX,
+    ModelY,
+    MultiTableDerived,
+    MyManager,
+    MyManagerQuerySet,
+    NonProxyChild,
+    One2OneRelatingModel,
+    One2OneRelatingModelDerived,
+    ParentModelWithManager,
+    PlainA,
+    PlainB,
+    PlainC,
+    PlainChildModelWithManager,
+    PlainMyManager,
+    PlainMyManagerQuerySet,
+    PlainParentModelWithManager,
+    ProxiedBase,
+    ProxyBase,
+    ProxyChild,
+    ProxyModelA,
+    ProxyModelB,
+    ProxyModelBase,
+    QuerySet,
+    RedheadDuck,
+    RelationA,
+    RelationB,
+    RelationBC,
+    RelationBase,
+    RubberDuck,
+    TestParentLinkAndRelatedName,
+    UUIDArtProject,
+    UUIDPlainA,
+    UUIDPlainB,
+    UUIDPlainC,
+    UUIDProject,
+    UUIDResearchProject,
+)
 
 
-class PolymorphicTests(TestCase):
+class PolymorphicTests(TransactionTestCase):
     """
     The test suite
     """
@@ -128,19 +204,23 @@ class PolymorphicTests(TestCase):
         Create the chain of objects of Model2,
         this is reused in various tests.
         """
-        Model2A.objects.create(field1='A1')
-        Model2B.objects.create(field1='B1', field2='B2')
-        Model2C.objects.create(field1='C1', field2='C2', field3='C3')
-        Model2D.objects.create(field1='D1', field2='D2', field3='D3', field4='D4')
+        a = Model2A.objects.create(field1='A1')
+        b = Model2B.objects.create(field1='B1', field2='B2')
+        c = Model2C.objects.create(field1='C1', field2='C2', field3='C3')
+        d = Model2D.objects.create(field1='D1', field2='D2', field3='D3', field4='D4')
+
+        return a, b, c, d
 
     def test_simple_inheritance(self):
         self.create_model2abcd()
 
-        objects = list(Model2A.objects.all())
-        self.assertEqual(repr(objects[0]), '<Model2A: id 1, field1 (CharField)>')
-        self.assertEqual(repr(objects[1]), '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
-        self.assertEqual(repr(objects[2]), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
-        self.assertEqual(repr(objects[3]), '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
+        objects = Model2A.objects.all()
+        self.assertQuerysetEqual(
+            objects,
+            [Model2A, Model2B, Model2C, Model2D],
+            transform=lambda o: o.__class__,
+            ordered=False,
+        )
 
     def test_defer_fields(self):
         self.create_model2abcd()
@@ -148,14 +228,14 @@ class PolymorphicTests(TestCase):
         objects_deferred = Model2A.objects.defer('field1')
 
         self.assertNotIn('field1', objects_deferred[0].__dict__, 'field1 was not deferred (using defer())')
-        self.assertEqual(repr(objects_deferred[0]),
-                         '<Model2A: id 1, field1 (CharField), deferred[field1]>')
-        self.assertEqual(repr(objects_deferred[1]),
-                         '<Model2B: id 2, field1 (CharField), field2 (CharField), deferred[field1]>')
-        self.assertEqual(repr(objects_deferred[2]),
-                         '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField), deferred[field1]>')
-        self.assertEqual(repr(objects_deferred[3]),
-                         '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField), deferred[field1]>')
+        self.assertRegex(repr(objects_deferred[0]),
+                         '<Model2A: id \d+, field1 \(CharField\), deferred\[field1\]>')
+        self.assertRegex(repr(objects_deferred[1]),
+                         '<Model2B: id \d+, field1 \(CharField\), field2 \(CharField\), deferred\[field1\]>')
+        self.assertRegex(repr(objects_deferred[2]),
+                         '<Model2C: id \d+, field1 \(CharField\), field2 \(CharField\), field3 \(CharField\), deferred\[field1\]>')
+        self.assertRegex(repr(objects_deferred[3]),
+                         '<Model2D: id \d+, field1 \(CharField\), field2 \(CharField\), field3 \(CharField\), field4 \(CharField\), deferred\[field1\]>')
 
         objects_only = Model2A.objects.only('pk', 'polymorphic_ctype', 'field1')
 
@@ -166,16 +246,33 @@ class PolymorphicTests(TestCase):
                       ' on a child model')
         self.assertNotIn('field4', objects_only[3].__dict__,
                          'field4 was not deferred (using only())')
-        self.assertEqual(repr(objects_only[0]),
-                         '<Model2A: id 1, field1 (CharField)>')
-        self.assertEqual(repr(objects_only[1]),
-                         '<Model2B: id 2, field1 (CharField), field2 (CharField), deferred[field2]>')
-        self.assertEqual(repr(objects_only[2]),
-                         '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField), '
-                         'deferred[field2,field3,model2a_ptr_id]>')
-        self.assertEqual(repr(objects_only[3]),
-                         '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField), '
-                         'deferred[field2,field3,field4,model2a_ptr_id,model2b_ptr_id]>')
+        self.assertRegex(repr(objects_only[0]),
+                         '<Model2A: id \d+, field1 \(CharField\)>')
+        self.assertRegex(repr(objects_only[1]),
+                         '<Model2B: id \d+, field1 \(CharField\), field2 \(CharField\), deferred\[field2\]>')
+        self.assertRegex(repr(objects_only[2]),
+                         '<Model2C: id \d+, field1 \(CharField\), field2 \(CharField\), field3 \(CharField\), '
+                         'deferred\[field2,field3,model2a_ptr_id\]>')
+        self.assertRegex(repr(objects_only[3]),
+                         '<Model2D: id \d+, field1 \(CharField\), field2 \(CharField\), field3 \(CharField\), field4 \(CharField\), '
+                         'deferred\[field2,field3,field4,model2a_ptr_id,model2b_ptr_id\]>')
+
+        ModelX.objects.create(field_b="A1", field_x="A2")
+        ModelY.objects.create(field_b="B1", field_y="B2")
+
+        objects_deferred = Base.objects.defer('ModelY___field_y')
+        self.assertRegex(repr(objects_deferred[0]),
+                         '<ModelX: id \d+, field_b \(CharField\), field_x \(CharField\)>')
+        self.assertRegex(repr(objects_deferred[1]),
+                         '<ModelY: id \d+, field_b \(CharField\), field_y \(CharField\), deferred\[field_y\]>')
+
+        objects_only = Base.objects.only(
+            'polymorphic_ctype', 'ModelY___field_y', 'ModelX___field_x',
+        )
+        self.assertRegex(repr(objects_only[0]),
+                         '<ModelX: id \d+, field_b \(CharField\), field_x \(CharField\), deferred\[field_b\]>')
+        self.assertRegex(repr(objects_only[1]),
+                         '<ModelY: id \d+, field_b \(CharField\), field_y \(CharField\), deferred\[field_b\]>')
 
     def test_defer_related_fields(self):
         self.create_model2abcd()
@@ -183,43 +280,36 @@ class PolymorphicTests(TestCase):
         objects_deferred_field4 = Model2A.objects.defer('Model2D___field4')
         self.assertNotIn('field4', objects_deferred_field4[3].__dict__,
                          'field4 was not deferred (using defer(), traversing inheritance)')
-        self.assertEqual(repr(objects_deferred_field4[0]),
-                         '<Model2A: id 1, field1 (CharField)>')
-        self.assertEqual(repr(objects_deferred_field4[1]),
-                         '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
-        self.assertEqual(repr(objects_deferred_field4[2]),
-                         '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
-        self.assertEqual(repr(objects_deferred_field4[3]),
-                         '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField), deferred[field4]>')
+        self.assertEqual(objects_deferred_field4[0].__class__, Model2A)
+        self.assertEqual(objects_deferred_field4[1].__class__, Model2B)
+        self.assertEqual(objects_deferred_field4[2].__class__, Model2C)
+        self.assertEqual(objects_deferred_field4[3].__class__, Model2D)
 
         objects_only_field4 = Model2A.objects.only(
             'polymorphic_ctype', 'field1',
             'Model2B___id', 'Model2B___field2', 'Model2B___model2a_ptr',
             'Model2C___id', 'Model2C___field3', 'Model2C___model2b_ptr',
             'Model2D___id', 'Model2D___model2c_ptr')
-        self.assertEqual(repr(objects_only_field4[0]),
-                         '<Model2A: id 1, field1 (CharField)>')
-        self.assertEqual(repr(objects_only_field4[1]),
-                         '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
-        self.assertEqual(repr(objects_only_field4[2]),
-                         '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
-        self.assertEqual(repr(objects_only_field4[3]),
-                         '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField), deferred[field4]>')
+        self.assertEqual(objects_only_field4[0].__class__, Model2A)
+        self.assertEqual(objects_only_field4[1].__class__, Model2B)
+        self.assertEqual(objects_only_field4[2].__class__, Model2C)
+        self.assertEqual(objects_only_field4[3].__class__, Model2D)
 
     def test_manual_get_real_instance(self):
         self.create_model2abcd()
 
         o = Model2A.objects.non_polymorphic().get(field1='C1')
-        self.assertEqual(repr(o.get_real_instance()), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
+        self.assertEqual(o.get_real_instance().__class__, Model2C)
 
     def test_non_polymorphic(self):
         self.create_model2abcd()
 
         objects = list(Model2A.objects.all().non_polymorphic())
-        self.assertEqual(repr(objects[0]), '<Model2A: id 1, field1 (CharField)>')
-        self.assertEqual(repr(objects[1]), '<Model2A: id 2, field1 (CharField)>')
-        self.assertEqual(repr(objects[2]), '<Model2A: id 3, field1 (CharField)>')
-        self.assertEqual(repr(objects[3]), '<Model2A: id 4, field1 (CharField)>')
+        self.assertQuerysetEqual(
+            objects,
+            [Model2A, Model2A, Model2A, Model2A],
+            transform=lambda o: o.__class__,
+        )
 
     def test_get_real_instances(self):
         self.create_model2abcd()
@@ -227,58 +317,79 @@ class PolymorphicTests(TestCase):
 
         # from queryset
         objects = qs.get_real_instances()
-        self.assertEqual(repr(objects[0]), '<Model2A: id 1, field1 (CharField)>')
-        self.assertEqual(repr(objects[1]), '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
-        self.assertEqual(repr(objects[2]), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
-        self.assertEqual(repr(objects[3]), '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
+        self.assertQuerysetEqual(
+            objects,
+            [Model2A, Model2B, Model2C, Model2D],
+            transform=lambda o: o.__class__,
+        )
 
         # from a manual list
         objects = Model2A.objects.get_real_instances(list(qs))
-        self.assertEqual(repr(objects[0]), '<Model2A: id 1, field1 (CharField)>')
-        self.assertEqual(repr(objects[1]), '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
-        self.assertEqual(repr(objects[2]), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
-        self.assertEqual(repr(objects[3]), '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
+        self.assertQuerysetEqual(
+            objects,
+            [Model2A, Model2B, Model2C, Model2D],
+            transform=lambda o: o.__class__,
+        )
 
     def test_translate_polymorphic_q_object(self):
         self.create_model2abcd()
 
         q = Model2A.translate_polymorphic_Q_object(Q(instance_of=Model2C))
         objects = Model2A.objects.filter(q)
-        self.assertEqual(repr(objects[0]), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
-        self.assertEqual(repr(objects[1]), '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
+        self.assertQuerysetEqual(
+            objects,
+            [Model2C, Model2D],
+            transform=lambda o: o.__class__,
+            ordered=False,
+        )
 
     def test_base_manager(self):
-        def show_base_manager(model):
-            return "{0} {1}".format(
-                repr(type(model._base_manager)),
-                repr(model._base_manager.model)
+        def base_manager(model):
+            return (
+                type(model._base_manager),
+                model._base_manager.model
             )
 
-        self.assertEqual(show_base_manager(PlainA), "<class 'django.db.models.manager.Manager'> <class 'polymorphic.tests.PlainA'>")
-        self.assertEqual(show_base_manager(PlainB), "<class 'django.db.models.manager.Manager'> <class 'polymorphic.tests.PlainB'>")
-        self.assertEqual(show_base_manager(PlainC), "<class 'django.db.models.manager.Manager'> <class 'polymorphic.tests.PlainC'>")
+        self.assertEqual(
+            base_manager(PlainA),
+            (models.Manager, PlainA),
+        )
+        self.assertEqual(
+            base_manager(PlainB),
+            (models.Manager, PlainB),
+        )
+        self.assertEqual(
+            base_manager(PlainC),
+            (models.Manager, PlainC),
+        )
 
-        self.assertEqual(show_base_manager(Model2A), "<class 'polymorphic.managers.PolymorphicManager'> <class 'polymorphic.tests.Model2A'>")
-        if django.VERSION >= (1, 10):
-            # The new inheritance makes all model levels polymorphic
-            self.assertEqual(show_base_manager(Model2B), "<class 'polymorphic.managers.PolymorphicManager'> <class 'polymorphic.tests.Model2B'>")
-            self.assertEqual(show_base_manager(Model2C), "<class 'polymorphic.managers.PolymorphicManager'> <class 'polymorphic.tests.Model2C'>")
-        else:
-            self.assertEqual(show_base_manager(Model2B), "<class 'django.db.models.manager.Manager'> <class 'polymorphic.tests.Model2B'>")
-            self.assertEqual(show_base_manager(Model2C), "<class 'django.db.models.manager.Manager'> <class 'polymorphic.tests.Model2C'>")
+        self.assertEqual(
+            base_manager(Model2A),
+            (PolymorphicManager, Model2A),
+        )
+        self.assertEqual(
+            base_manager(Model2B),
+            (PolymorphicManager, Model2B),
+        )
+        self.assertEqual(
+            base_manager(Model2C),
+            (PolymorphicManager, Model2C),
+        )
 
-        self.assertEqual(show_base_manager(One2OneRelatingModel), "<class 'polymorphic.managers.PolymorphicManager'> <class 'polymorphic.tests.One2OneRelatingModel'>")
-        if django.VERSION >= (1, 10):
-            # The new inheritance makes all model levels polymorphic
-            self.assertEqual(show_base_manager(One2OneRelatingModelDerived), "<class 'polymorphic.managers.PolymorphicManager'> <class 'polymorphic.tests.One2OneRelatingModelDerived'>")
-        else:
-            self.assertEqual(show_base_manager(One2OneRelatingModelDerived), "<class 'django.db.models.manager.Manager'> <class 'polymorphic.tests.One2OneRelatingModelDerived'>")
+        self.assertEqual(
+            base_manager(One2OneRelatingModel),
+            (PolymorphicManager, One2OneRelatingModel),
+        )
+        self.assertEqual(
+            base_manager(One2OneRelatingModelDerived),
+            (PolymorphicManager, One2OneRelatingModelDerived),
+        )
 
     def test_instance_default_manager(self):
-        def show_default_manager(instance):
-            return "{0} {1}".format(
-                repr(type(instance.__class__._default_manager)),
-                repr(instance.__class__._default_manager.model)
+        def default_manager(instance):
+            return (
+                type(instance.__class__._default_manager),
+                instance.__class__._default_manager.model
             )
 
         plain_a = PlainA(field1='C1')
@@ -289,22 +400,40 @@ class PolymorphicTests(TestCase):
         model_2b = Model2B(field2='C1')
         model_2c = Model2C(field3='C1')
 
-        self.assertEqual(show_default_manager(plain_a), "<class 'django.db.models.manager.Manager'> <class 'polymorphic.tests.PlainA'>")
-        self.assertEqual(show_default_manager(plain_b), "<class 'django.db.models.manager.Manager'> <class 'polymorphic.tests.PlainB'>")
-        self.assertEqual(show_default_manager(plain_c), "<class 'django.db.models.manager.Manager'> <class 'polymorphic.tests.PlainC'>")
+        self.assertEqual(
+            default_manager(plain_a),
+            (models.Manager, PlainA),
+        )
+        self.assertEqual(
+            default_manager(plain_b),
+            (models.Manager, PlainB),
+        )
+        self.assertEqual(
+            default_manager(plain_c),
+            (models.Manager, PlainC),
+        )
 
-        self.assertEqual(show_default_manager(model_2a), "<class 'polymorphic.managers.PolymorphicManager'> <class 'polymorphic.tests.Model2A'>")
-        self.assertEqual(show_default_manager(model_2b), "<class 'polymorphic.managers.PolymorphicManager'> <class 'polymorphic.tests.Model2B'>")
-        self.assertEqual(show_default_manager(model_2c), "<class 'polymorphic.managers.PolymorphicManager'> <class 'polymorphic.tests.Model2C'>")
+        self.assertEqual(
+            default_manager(model_2a),
+            (PolymorphicManager, Model2A),
+        )
+        self.assertEqual(
+            default_manager(model_2b),
+            (PolymorphicManager, Model2B),
+        )
+        self.assertEqual(
+            default_manager(model_2c),
+            (PolymorphicManager, Model2C),
+        )
 
     def test_foreignkey_field(self):
         self.create_model2abcd()
 
         object2a = Model2A.base_objects.get(field1='C1')
-        self.assertEqual(repr(object2a.model2b), '<Model2B: id 3, field1 (CharField), field2 (CharField)>')
+        self.assertEqual(object2a.model2b.__class__, Model2B)
 
         object2b = Model2B.base_objects.get(field1='C1')
-        self.assertEqual(repr(object2b.model2c), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
+        self.assertEqual(object2b.model2c.__class__, Model2C)
 
     def test_onetoone_field(self):
         self.create_model2abcd()
@@ -313,11 +442,15 @@ class PolymorphicTests(TestCase):
         b = One2OneRelatingModelDerived.objects.create(one2one=a, field1='f1', field2='f2')
 
         # this result is basically wrong, probably due to Django cacheing (we used base_objects), but should not be a problem
-        self.assertEqual(repr(b.one2one), '<Model2A: id 3, field1 (CharField)>')
+        self.assertEqual(b.one2one.__class__, Model2A)
+        self.assertEqual(b.one2one_id, b.one2one.id)
 
         c = One2OneRelatingModelDerived.objects.get(field1='f1')
-        self.assertEqual(repr(c.one2one), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
-        self.assertEqual(repr(a.one2onerelatingmodel), '<One2OneRelatingModelDerived: One2OneRelatingModelDerived object>')
+        self.assertEqual(c.one2one.__class__, Model2C)
+        self.assertEqual(
+            a.one2onerelatingmodel.__class__,
+            One2OneRelatingModelDerived,
+        )
 
     def test_manytomany_field(self):
         # Model 1
@@ -344,19 +477,32 @@ class PolymorphicTests(TestCase):
         # no pretty printing
         ModelShow1_plain.objects.create(field1='abc')
         ModelShow2_plain.objects.create(field1='abc', field2='def')
-        self.assertEqual(qrepr(ModelShow1_plain.objects.all()), '<PolymorphicQuerySet [<ModelShow1_plain: ModelShow1_plain object>, <ModelShow2_plain: ModelShow2_plain object>]>')
+        self.assertQuerysetEqual(
+            ModelShow1_plain.objects.all(),
+            [ModelShow1_plain, ModelShow2_plain],
+            transform=lambda o: o.__class__,
+            ordered=False,
+        )
 
     def test_extra_method(self):
-        self.create_model2abcd()
+        a, b, c, d = self.create_model2abcd()
 
-        objects = list(Model2A.objects.extra(where=['id IN (2, 3)']))
-        self.assertEqual(repr(objects[0]), '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
-        self.assertEqual(repr(objects[1]), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
+        objects = Model2A.objects.extra(
+            where=['id IN ({}, {})'.format(b.id, c.id)]
+        )
+        self.assertQuerysetEqual(
+            objects,
+            [Model2B, Model2C],
+            transform=lambda o: o.__class__,
+            ordered=False,
+        )
 
         objects = Model2A.objects.extra(select={"select_test": "field1 = 'A1'"}, where=["field1 = 'A1' OR field1 = 'B1'"], order_by=['-id'])
-        self.assertEqual(repr(objects[0]), '<Model2B: id 2, field1 (CharField), field2 (CharField) - Extra: select_test (int)>')
-        self.assertEqual(repr(objects[1]), '<Model2A: id 1, field1 (CharField) - Extra: select_test (int)>')
-        self.assertEqual(len(objects), 2)   # Placed after the other tests, only verifying whether there are no more additional objects.
+        self.assertQuerysetEqual(
+            objects,
+            [Model2B, Model2A],
+            transform=lambda o: o.__class__,
+        )
 
         ModelExtraA.objects.create(field1='A1')
         ModelExtraB.objects.create(field1='B1', field2='B2')
@@ -364,7 +510,11 @@ class PolymorphicTests(TestCase):
         ModelExtraExternal.objects.create(topic='extra1')
         ModelExtraExternal.objects.create(topic='extra2')
         ModelExtraExternal.objects.create(topic='extra3')
-        objects = ModelExtraA.objects.extra(tables=["polymorphic_modelextraexternal"], select={"topic": "polymorphic_modelextraexternal.topic"}, where=["polymorphic_modelextraa.id = polymorphic_modelextraexternal.id"])
+        objects = ModelExtraA.objects.extra(
+            tables=["tests_modelextraexternal"],
+            select={"topic": "tests_modelextraexternal.topic"},
+            where=["tests_modelextraa.id = tests_modelextraexternal.id"],
+        )
         if six.PY3:
             self.assertEqual(repr(objects[0]), '<ModelExtraA: id 1, field1 (CharField) "A1" - Extra: topic (str) "extra1">')
             self.assertEqual(repr(objects[1]), '<ModelExtraB: id 2, field1 (CharField) "B1", field2 (CharField) "B2" - Extra: topic (str) "extra2">')
@@ -379,34 +529,59 @@ class PolymorphicTests(TestCase):
         self.create_model2abcd()
 
         objects = Model2A.objects.instance_of(Model2B)
-        self.assertEqual(repr(objects[0]), '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
-        self.assertEqual(repr(objects[1]), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
-        self.assertEqual(repr(objects[2]), '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
-        self.assertEqual(len(objects), 3)
+        self.assertQuerysetEqual(
+            objects,
+            [Model2B, Model2C, Model2D],
+            transform=lambda o: o.__class__,
+            ordered=False,
+        )
 
         objects = Model2A.objects.filter(instance_of=Model2B)
-        self.assertEqual(repr(objects[0]), '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
-        self.assertEqual(repr(objects[1]), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
-        self.assertEqual(repr(objects[2]), '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
-        self.assertEqual(len(objects), 3)
+        self.assertQuerysetEqual(
+            objects,
+            [Model2B, Model2C, Model2D],
+            transform=lambda o: o.__class__,
+            ordered=False,
+        )
 
         objects = Model2A.objects.filter(Q(instance_of=Model2B))
-        self.assertEqual(repr(objects[0]), '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
-        self.assertEqual(repr(objects[1]), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
-        self.assertEqual(repr(objects[2]), '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
-        self.assertEqual(len(objects), 3)
+        self.assertQuerysetEqual(
+            objects,
+            [Model2B, Model2C, Model2D],
+            transform=lambda o: o.__class__,
+            ordered=False,
+        )
 
         objects = Model2A.objects.not_instance_of(Model2B)
-        self.assertEqual(repr(objects[0]), '<Model2A: id 1, field1 (CharField)>')
-        self.assertEqual(len(objects), 1)
+        self.assertQuerysetEqual(
+            objects,
+            [Model2A],
+            transform=lambda o: o.__class__,
+            ordered=False,
+        )
 
     def test_polymorphic___filter(self):
         self.create_model2abcd()
 
         objects = Model2A.objects.filter(Q(Model2B___field2='B2') | Q(Model2C___field3='C3'))
-        self.assertEqual(len(objects), 2)
-        self.assertEqual(repr(objects[0]), '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
-        self.assertEqual(repr(objects[1]), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
+        self.assertQuerysetEqual(
+            objects,
+            [Model2B, Model2C],
+            transform=lambda o: o.__class__,
+            ordered=False,
+        )
+
+    def test_polymorphic_applabel___filter(self):
+        self.create_model2abcd()
+
+        assert Model2B._meta.app_label == 'tests'
+        objects = Model2A.objects.filter(Q(tests__Model2B___field2='B2') | Q(tests__Model2C___field3='C3'))
+        self.assertQuerysetEqual(
+            objects,
+            [Model2B, Model2C],
+            transform=lambda o: o.__class__,
+            ordered=False,
+        )
 
     def test_query_filter_exclude_is_immutable(self):
         # given
@@ -442,18 +617,20 @@ class PolymorphicTests(TestCase):
         self.assertEqual(len(objects), 1)
 
     def test_delete(self):
-        self.create_model2abcd()
+        a, b, c, d = self.create_model2abcd()
 
-        oa = Model2A.objects.get(id=2)
-        self.assertEqual(repr(oa), '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
+        oa = Model2A.objects.get(id=b.id)
+        self.assertEqual(oa.__class__, Model2B)
         self.assertEqual(Model2A.objects.count(), 4)
 
         oa.delete()
         objects = Model2A.objects.all()
-        self.assertEqual(repr(objects[0]), '<Model2A: id 1, field1 (CharField)>')
-        self.assertEqual(repr(objects[1]), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
-        self.assertEqual(repr(objects[2]), '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
-        self.assertEqual(len(objects), 3)
+        self.assertQuerysetEqual(
+            objects,
+            [Model2A, Model2C, Model2D],
+            transform=lambda o: o.__class__,
+            ordered=False,
+        )
 
     def test_combine_querysets(self):
         ModelX.objects.create(field_x='x')
@@ -514,9 +691,14 @@ class PolymorphicTests(TestCase):
         ModelWithMyManager.objects.create(field1='D1b', field4='D4b')
 
         objects = ModelWithMyManager.objects.all()   # MyManager should reverse the sorting of field1
-        self.assertEqual(repr(objects[0]), '<ModelWithMyManager: id 6, field1 (CharField) "D1b", field4 (CharField) "D4b">')
-        self.assertEqual(repr(objects[1]), '<ModelWithMyManager: id 5, field1 (CharField) "D1a", field4 (CharField) "D4a">')
-        self.assertEqual(len(objects), 2)
+        self.assertQuerysetEqual(
+            objects,
+            [
+                (ModelWithMyManager, 'D1b', 'D4b'),
+                (ModelWithMyManager, 'D1a', 'D4a'),
+            ],
+            transform=lambda o: (o.__class__, o.field1, o.field4),
+        )
 
         self.assertIs(type(ModelWithMyManager.objects), MyManager)
         self.assertIs(type(ModelWithMyManager._default_manager), MyManager)
@@ -528,9 +710,14 @@ class PolymorphicTests(TestCase):
         ModelWithMyManagerNoDefault.objects.create(field1='D1b', field4='D4b')
 
         objects = ModelWithMyManagerNoDefault.my_objects.all()   # MyManager should reverse the sorting of field1
-        self.assertEqual(repr(objects[0]), '<ModelWithMyManagerNoDefault: id 6, field1 (CharField) "D1b", field4 (CharField) "D4b">')
-        self.assertEqual(repr(objects[1]), '<ModelWithMyManagerNoDefault: id 5, field1 (CharField) "D1a", field4 (CharField) "D4a">')
-        self.assertEqual(len(objects), 2)
+        self.assertQuerysetEqual(
+            objects,
+            [
+                (ModelWithMyManagerNoDefault, 'D1b', 'D4b'),
+                (ModelWithMyManagerNoDefault, 'D1a', 'D4a'),
+            ],
+            transform=lambda o: (o.__class__, o.field1, o.field4),
+        )
 
         self.assertIs(type(ModelWithMyManagerNoDefault.my_objects), MyManager)
         self.assertIs(type(ModelWithMyManagerNoDefault.objects), PolymorphicManager)
@@ -553,9 +740,15 @@ class PolymorphicTests(TestCase):
         ModelWithMyManager2.objects.create(field1='D1b', field4='D4b')
 
         objects = ModelWithMyManager2.objects.all()
-        self.assertEqual(repr(objects[0]), '<ModelWithMyManager2: id 5, field1 (CharField) "D1a", field4 (CharField) "D4a">')
-        self.assertEqual(repr(objects[1]), '<ModelWithMyManager2: id 6, field1 (CharField) "D1b", field4 (CharField) "D4b">')
-        self.assertEqual(len(objects), 2)
+        self.assertQuerysetEqual(
+            objects,
+            [
+                (ModelWithMyManager2, 'D1a', 'D4a'),
+                (ModelWithMyManager2, 'D1b', 'D4b'),
+            ],
+            transform=lambda o: (o.__class__, o.field1, o.field4),
+            ordered=False,
+        )
 
         self.assertEqual(type(ModelWithMyManager2.objects).__name__, 'PolymorphicManagerFromMyManagerQuerySet')
         self.assertEqual(type(ModelWithMyManager2._default_manager).__name__, 'PolymorphicManagerFromMyManagerQuerySet')
@@ -565,17 +758,6 @@ class PolymorphicTests(TestCase):
         # by choice of MRO, should be MyManager from MROBase1.
         self.assertIs(type(MRODerived.objects), MyManager)
 
-        if django.VERSION < (1, 10, 1):
-            # The change for https://code.djangoproject.com/ticket/27073
-            # in https://github.com/django/django/commit/d4eefc7e2af0d93283ed1c03e0af0a482982b6f0
-            # removes the assignment to _default_manager
-
-            # check for correct default manager
-            self.assertIs(type(MROBase1._default_manager), MyManager)
-
-            # Django vanilla inheritance does not inherit MyManager as _default_manager here
-            self.assertIs(type(MROBase2._default_manager), MyManager)
-
     def test_queryset_assignment(self):
         # This is just a consistency check for now, testing standard Django behavior.
         parent = PlainParentModelWithManager.objects.create()
@@ -766,36 +948,52 @@ class PolymorphicTests(TestCase):
         result = Model2B.objects.annotate(val=Concat('field1', 'field2'))
         self.assertEqual(list(result), [])
 
-    def test_contrib_guardian(self):
-        # Regular Django inheritance should return the child model content type.
-        obj = PlainC()
-        ctype = get_polymorphic_base_content_type(obj)
-        self.assertEqual(ctype.name, 'plain c')
+    def test_null_polymorphic_id(self):
+        """Test that a proper error message is displayed when the database lacks the ``polymorphic_ctype_id``"""
+        Model2A.objects.create(field1='A1')
+        Model2B.objects.create(field1='A1', field2='B2')
+        Model2B.objects.create(field1='A1', field2='B2')
+        Model2A.objects.all().update(polymorphic_ctype_id=None)
 
-        ctype = get_polymorphic_base_content_type(PlainC)
-        self.assertEqual(ctype.name, 'plain c')
+        with self.assertRaises(PolymorphicTypeUndefined):
+            list(Model2A.objects.all())
 
-        # Polymorphic inheritance should return the parent model content type.
-        obj = Model2D()
-        ctype = get_polymorphic_base_content_type(obj)
-        self.assertEqual(ctype.name, 'model2a')
+    def test_bulk_create_abstract_inheritance(self):
+        ArtProject.objects.bulk_create([
+            ArtProject(topic='Painting with Tim', artist='T. Turner'),
+            ArtProject(topic='Sculpture with Tim', artist='T. Turner'),
+        ])
+        self.assertEqual(
+            sorted(ArtProject.objects.values_list('topic', 'artist')),
+            [('Painting with Tim', 'T. Turner'), ('Sculpture with Tim', 'T. Turner')]
+        )
 
-        ctype = get_polymorphic_base_content_type(Model2D)
-        self.assertEqual(ctype.name, 'model2a')
+    def test_bulk_create_proxy_inheritance(self):
+        RedheadDuck.objects.bulk_create([
+            RedheadDuck(name='redheadduck1'),
+            Duck(name='duck1'),
+            RubberDuck(name='rubberduck1'),
+        ])
+        RubberDuck.objects.bulk_create([
+            RedheadDuck(name='redheadduck2'),
+            RubberDuck(name='rubberduck2'),
+            Duck(name='duck2'),
+        ])
+        self.assertEqual(
+            sorted(RedheadDuck.objects.values_list('name', flat=True)),
+            ['redheadduck1', 'redheadduck2'],
+        )
+        self.assertEqual(
+            sorted(RubberDuck.objects.values_list('name', flat=True)),
+            ['rubberduck1', 'rubberduck2'],
+        )
+        self.assertEqual(
+            sorted(Duck.objects.values_list('name', flat=True)),
+            ['duck1', 'duck2', 'redheadduck1', 'redheadduck2', 'rubberduck1', 'rubberduck2'],
+        )
 
-
-def qrepr(data):
-    """
-    Ensure consistent repr() output for the QuerySet object.
-    """
-    if isinstance(data, QuerySet):
-        if django.VERSION >= (1, 11):
-            return repr(data)
-        elif django.VERSION >= (1, 10):
-            # Django 1.11 still shows "<QuerySet [", not taking the actual type into account.
-            return '<{0} {1}'.format(data.__class__.__name__, repr(data)[10:])
-        else:
-            # Simulate Django 1.11 behavior for older Django versions.
-            return '<{0} {1}>'.format(data.__class__.__name__, repr(data))
-
-    return repr(data)
+    def test_bulk_create_unsupported_multi_table_inheritance(self):
+        with self.assertRaises(ValueError):
+            MultiTableDerived.objects.bulk_create([
+                MultiTableDerived(field1='field1', field2='field2')
+            ])
